'use strict';

var worker_threads = require('worker_threads');
var pluginCommonjs = require('@rollup/plugin-commonjs');
var pluginJson = require('@rollup/plugin-json');
var pluginReplace = require('@rollup/plugin-replace');
var pluginNodeResolve = require('@rollup/plugin-node-resolve');
var pirates = require('pirates');
var fs = require('fs');
var isModule = require('is-module');
var module$1 = require('module');
var path = require('path');
var sucrase = require('sucrase');
var rollup$1 = require('rollup');
var pluginVirtual = require('@rollup/plugin-virtual');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var worker_threads__default = /*#__PURE__*/_interopDefaultLegacy(worker_threads);
var pluginCommonjs__default = /*#__PURE__*/_interopDefaultLegacy(pluginCommonjs);
var pluginJson__default = /*#__PURE__*/_interopDefaultLegacy(pluginJson);
var pluginReplace__default = /*#__PURE__*/_interopDefaultLegacy(pluginReplace);
var pluginNodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(pluginNodeResolve);
var pirates__default = /*#__PURE__*/_interopDefaultLegacy(pirates);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var isModule__default = /*#__PURE__*/_interopDefaultLegacy(isModule);
var module__default = /*#__PURE__*/_interopDefaultLegacy(module$1);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var sucrase__default = /*#__PURE__*/_interopDefaultLegacy(sucrase);
var rollup__default = /*#__PURE__*/_interopDefaultLegacy(rollup$1);
var pluginVirtual__default = /*#__PURE__*/_interopDefaultLegacy(pluginVirtual);

let resolvePlugin = pluginNodeResolve__default['default'];

if ('default' in resolvePlugin) {
  // @ts-ignore
  resolvePlugin = resolvePlugin.default;
}

const plugins = [
  // @ts-ignore
  pluginReplace__default['default']({
    'process.env.NODE_ENV': `"${process.env.NODE_ENV}"` || '"development"',
  }),
  // @ts-ignore
  resolvePlugin({
    mainFields: ['module', 'browser', 'main'],
  }),
  // @ts-ignore
  pluginJson__default['default'](),
  // @ts-ignore
  pluginCommonjs__default['default']({
    sourceMap: false,
  }),
];

/**
 * Retrieve the default Rollup-config
 * @returns { import("rollup").RollupOptions }
 */
var getDefaultRollupConfig = function getDefaultRollupConfig() {
  return {
    // Only bundle local package files
    external: (id, parent, isResolved) => {
      // Skip if already handled by plugin
      if (isResolved || (parent && parent.includes('?commonjs-proxy'))) {
        return false;
      }
      return /^[^./\0]/.test(id);
    },
    plugins: plugins.slice(),
    treeshake: false,
    output: {
      format: 'es',
      sourcemap: false,
    },
  };
};

var defaultRollupConfig = {
	getDefaultRollupConfig: getDefaultRollupConfig
};

function createCommonjsModule(fn, basedir, module) {
	return module = {
	  path: basedir,
	  exports: {},
	  require: function (path, base) {
      return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
    }
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var module_1 = createCommonjsModule(function (module) {

const { addHook } = pirates__default['default'];

// @ts-ignore

// Work around @rollup/plugin-commonjs dynamic require
// @ts-ignore
const loadModule = module__default['default']._load;



const IMPORT_EXTS = ['.js', '.mjs'];
const IMPORT_EXTS_TRANSPILER = ['.js', '.jsx', '.mjs', '.ts', '.tsx'];
// Sync with config.js#extensionsByType
const JS_EXTENSIONS = ['.ts', '.tsx', '.mjs', '.jsx', '.js', '.json'];
const RE_TRANSPILER_HANDLES_SERVER = /\(\s?[a-zA-Z]+,\s?[a-zA-Z]+\s?\)/;

/** @type { () => void } */
let revertHook;

module.exports = {
  isModule,
  importModule,
};

/**
 * Determine if filePath or code is es module
 *
 * @param { string } filePathOrCode
 * @returns { boolean }
 */
function isModule(filePathOrCode) {
  if (
    !filePathOrCode.includes('\n') &&
    JS_EXTENSIONS.includes(path__default['default'].extname(filePathOrCode))
  ) {
    filePathOrCode = fs__default['default'].readFileSync(filePathOrCode, 'utf8');
  }
  return isModule__default['default'](filePathOrCode);
}

/**
 * Import esm/cjs module, transpiling if necessary (via require hook)
 *
 * @param { string } modulePath
 * @param { Transpiler } [transpiler]
 * @returns { any }
 */
function importModule(modulePath, transpiler) {
  if (revertHook !== undefined) {
    revertHook();
  }

  revertHook = addHook(
    (code, filePath) => {
      // Determine if transpiler supports transpiling server modules by checking number of arguments (filePath, isServer) handled
      if (
        transpiler !== undefined &&
        RE_TRANSPILER_HANDLES_SERVER.test(transpiler.toString())
      ) {
        const transpiled = transpiler(filePath, true);

        if (transpiled !== undefined) {
          // Ignore async
          if (!(transpiled instanceof Promise)) {
            code = transpiled;
          }
        }
      }

      if (!isModule(code)) {
        return code;
      }

      return sucrase__default['default'].transform(code, {
        transforms: ['imports'],
        filePath,
      }).code;
    },
    {
      exts: transpiler ? IMPORT_EXTS_TRANSPILER : IMPORT_EXTS,
      ignoreNodeModules: false,
    },
  );
  let mod = loadModule(modulePath, module, false);

  // Return default if only exported key
  if ('default' in mod && Object.keys(mod).length === 1) {
    mod = mod.default;
  }

  return mod;
}
});

const {
  isMainThread,
  parentPort,
  Worker,
  workerData,
} = worker_threads__default['default'];
const {
  getDefaultRollupConfig: getDefaultRollupConfig$1,
} = defaultRollupConfig;
const { importModule } = module_1;
const { rollup } = rollup__default['default'];
// @ts-ignore


class BundleWorker {
  /**
   * Constructor
   *
   * @param { string } [rollupConfigPath]
   */
  constructor(rollupConfigPath) {
    const defaultConfig = getDefaultRollupConfig$1();

    this.rollupConfig = rollupConfigPath
      ? mergeRollupConfig(defaultConfig, importModule(rollupConfigPath))
      : defaultConfig;
    this.worker;

    if (isMainThread) {
      // Load this file in Worker thread, passing `rollupConfigPath` as data
      this.worker = new Worker(__filename, { workerData: rollupConfigPath });
    }
  }

  /**
   * Bundle file at "inputPath"
   *
   * @param { string } inputPath
   * @param { string } outputPath
   * @param { string } sourcePrefix
   * @param { Array<string> } [namedExports]
   */
  async bundle(inputPath, outputPath, sourcePrefix, namedExports) {
    if (this.worker) {
      return new Promise((resolve, reject) => {
        this.worker.postMessage({
          inputPath,
          outputPath,
          sourcePrefix,
          namedExports,
        });
        this.worker.once('message', (err) => {
          err ? reject(err) : resolve();
        });
      });
    }

    const banner = sourcePrefix + inputPath;
    const parsedOptions = parseRollupOptions(banner, this.rollupConfig);
    const inputOptions = {
      ...parsedOptions.input,
      input: inputPath,
    };
    const outputOptions = {
      ...parsedOptions.output,
      file: outputPath,
    };

    if (namedExports && inputOptions.plugins) {
      inputOptions.plugins.unshift(
        pluginVirtual__default['default']({
          entry: `import entry from '${inputPath}';
        export default entry;
        export {${namedExports.join(', ')}} from '${inputPath}';`,
        }),
      );
      inputOptions.input = 'entry';
    }

    const bundled = await rollup(inputOptions);
    await bundled.write(outputOptions);
  }

  /**
   * Destroy instance
   *
   * @returns { Promise<void> }
   */
  destroy() {
    if (this.worker) {
      // @ts-ignore
      return this.worker.terminate();
    }
    return Promise.resolve();
  }
}

var bundleWorker = BundleWorker;

// Create instance if run in Worker thread
if (!isMainThread && parentPort) {
  /** @type { string | undefined } */
  const rollupConfigPath = workerData;
  const bundleWorker = new BundleWorker(rollupConfigPath);

  parentPort.on('message', async (
    /** @type { BundleWorkerMessage } */ message,
  ) => {
    const { inputPath, outputPath, sourcePrefix, namedExports } = message;

    try {
      await bundleWorker.bundle(
        inputPath,
        outputPath,
        sourcePrefix,
        namedExports,
      );
      parentPort.postMessage(false);
    } catch (err) {
      parentPort.postMessage(err);
    }
  });
}

/**
 * Merge user rollup.config with default
 *
 * @param { RollupOptions } defaultConfig
 * @param { RollupOptions } newConfig
 * @returns { RollupOptions }
 */
function mergeRollupConfig(defaultConfig, newConfig) {
  const {
    output: requiredOutput,
    plugins: defaultPlugins = [],
    ...defaultOptions
  } = defaultConfig;
  const { output, plugins = [], ...options } = newConfig;
  /** @type { { [name: string]: import('rollup').Plugin }}  */
  const newPluginsByName = plugins.reduce((newPluginsByName, plugin) => {
    // @ts-ignore
    newPluginsByName[plugin.name] = plugin;
    return newPluginsByName;
  }, {});
  let mergedPlugins = [];

  // Replace default plugin with new if it has the same name
  for (const plugin of defaultPlugins) {
    const { name } = plugin;
    mergedPlugins.push(
      name in newPluginsByName ? newPluginsByName[name] : plugin,
    );
  }

  return {
    ...defaultOptions,
    ...options,
    plugins: mergedPlugins,
    output: { ...output, ...requiredOutput },
  };
}

/**
 * Parse Rollup options
 *
 * @param { string } banner
 * @param { RollupOptions } options
 * @returns { { input: RollupInputOptions, output: RollupOutputOptions } }
 */
function parseRollupOptions(banner, options) {
  if (!options) {
    return { input: {}, output: { banner } };
  }

  let { input, treeshake, output = {}, watch, ...inputOverride } = options;

  if (Array.isArray(output)) {
    output = output[0];
  }

  const { file, format, sourcemap, ...outputOverride } = output;

  if ('banner' in outputOverride) {
    outputOverride.banner = banner + outputOverride.banner;
  } else {
    outputOverride.banner = banner;
  }

  // Shallow copy plugins
  if (inputOverride.plugins) {
    inputOverride.plugins = [...inputOverride.plugins];
  }

  return { input: inputOverride, output: outputOverride };
}

module.exports = bundleWorker;
